#include <ESP8266WiFi.h>
#include <EEPROM.h>
#include "ConfigService.h"
#include "Utils.h"

static const PROGMEM char okHeader[]="HTTP/1.1 200 OK\r\nContent-Type: text/html\r\n\r\n";

/** Config settings in EEProm (at address 0) */
EEPromData eePromData;

ConfigService::ConfigService(uint16_t port) 
: server(port)
{
    connIndex=0;
    stopAt=0;
    this->port=port;
    
    // Read EEPROM
    EEPROM.begin(sizeof(EEPromData));
    EEPROM.get(0,eePromData);
    eePromData.webserver[sizeof(eePromData.webserver)-1]=0;
    EEPROM.end();
};


void ConfigService::begin(unsigned int minutes, const char* apName, const char* password)
{
    // Switch the temporary access point on
    WiFi.mode(WIFI_AP_STA);
    WiFi.softAP(apName, password);
    server.begin();
    
    // Calculate when to disable the configuration service
    if (minutes>0)
    {
        stopAt=millis()+minutes*60*1000;
    }
    else
    {
        stopAt=0;
    }
}


void ConfigService::stop()
{
    server.stop();
    WiFi.mode(WIFI_STA);
}


void ConfigService::run()
{       
    if (server.status() != CLOSED)
    {    
        // Accept new connection, if present
        WiFiClient client = server.available();
        if (client)
        {
            for (int i = 0; i<MAX_TCP_CONNECTIONS; i++)
            {                    
                if (!connections[i].client.connected())
                {
                    connections[i].setup(client);
                    return;
                }                    
            }
            client.stop();
        }

        // Process one connection in each iterartion
        connections[connIndex].run();
        if (++connIndex >= MAX_TCP_CONNECTIONS) 
        {
            connIndex=0;
        }
        
        // Disable after some minutes
        if (stopAt>0 && millis() > stopAt && server.status()!=CLOSED)
        {
            server.close();
            WiFi.mode(WIFI_STA);
            stopAt=0;
        }
    }   
}
    
ConfigService::ConnectionHandler::ConnectionHandler()
{
    buffer[0]=0;
    state=WAIT_REQUEST;
}

void ConfigService::ConnectionHandler::setup(WiFiClient& client)
{
    this->client=client;
    buffer[0]=0;
    state=WAIT_REQUEST;   
}

void ConfigService::ConnectionHandler::run()
{
    if (client.connected())
    {       
        switch (state)
        {
            case WAIT_REQUEST:
                // Wait for a HTTP GET Request.            
                if (appendUntil(client, buffer, sizeof(buffer), '\n'))
                {
                    // If the line is a GET request
                    if (strncmp(buffer,"GET ",4)==0)
                    {
                        state=WAIT_EMPTY_LINE;
                    }
                    else
                    {
                        // Prepare to read the next line
                        buffer[0]=0;
                    }
                }
                break;

            case WAIT_EMPTY_LINE:
                if (flushUntil(client, '\n'))
                {
                    state=PROCESS_REQUEST;
                }
                break;

            case PROCESS_REQUEST:   
                // Find the ssid and the password parameters
                char* ssidPtr=strstr(buffer,"ssid=");
                char* passPtr=strstr(buffer,"pass=");
                char* serverPtr=strstr(buffer,"server=");
                
                // If the request provides both ssid and password, then configure the device
                if (ssidPtr && passPtr && serverPtr)
                {                   
                    // Decode the URL parameters
                    char ssidValue[32];
                    urlDecode(ssidValue,sizeof(ssidValue),ssidPtr+5);
                        
                    char passValue[32];
                    urlDecode(passValue,sizeof(passValue),passPtr+5);
                    
                    urlDecode(eePromData.webserver,sizeof(eePromData.webserver),serverPtr+7);
                    
                    // Apply the new settings
                    WiFi.begin(ssidValue, passValue);
                    WiFi.reconnect();
                    
                    // Write EEPROM
                    EEPROM.begin(sizeof(eePromData));
                    EEPROM.put(0,eePromData);
                    EEPROM.end();
                    
                    // Send HTTP response
                    client.print(FPSTR(okHeader));
                    client.print(F("<html><body>The settings are applied</body></html>"));
                }
                // else return the HTML form
                else
                {
                    client.print(FPSTR(okHeader));
                    client.print(F(
                        "<html><body><form method='get'>"
                        "Enter data of your access point and name or IP of the webserver to check:"
                        "<table border='0'>"
                        "<tr><td>SSID:</td><td><input name='ssid' value='"));
                    client.print(WiFi.SSID());
                    client.print(F("'></td></tr>"
                        "<tr><td>Password:</td><td><input name='pass'></td></tr>"
                        "<tr><td>Webserver:</td><td><input name='server' value='"));
                    client.print(eePromData.webserver);
                    client.print(F("'></td></tr>"
                        "<tr><td colspan='2' align='right'><input type='submit'></td></tr>"
                        "</table>"
                        "</form></body></html>"
                    ));
                }
                
                // Prepare for the next request
                client.stop();
                state=WAIT_REQUEST;
                buffer[0]=0;
                break;
        }
    }
}

    
