/**
 * WLAN monitor with ESP-01 Module.
 * 
 * Checks the connection to a webserver and displays the status on a 3-color LED.
 * 
 * Green  = Connection is fast
 * Blue   = Connection is slow
 * Red    = Connection failed (after 5 seconds timeout)
 * Off    = No connection to the AP
 * 
 * Within the first 5 minutes after power-up, the module operates as an access
 * point with name "WiFi Monitor". Connect to it and open the  website 
 * http://192.168.4.1/ to open the configuration form. There you can enter the 
 * name and password of your normal internet access point as well as the hostname or 
 * IP address of the target webserver that shall be checked.
 * 
 * Schematics:
 * The ESP-01 module is conected to a 3,3V power supply. The RGB LED is connected to:
 * 
 *             LED   3x 220 Ohm
 * VCC o---+---|>|---[===]-------o GPIO2 blue (=TxD)
 *         |
 *         +---|>|---[===]-------o GPIO1 red
 *         |
 *         +---|>|---[===]---+---o GPIO0 green
 *                    ____   |
 * GND |---------------  ----+
 *                 push-button
 * 
 * The push button is needed to enter firmware-upgrade mode.
 */

#include <ESP8266WiFi.h>
#include "ConfigService.h"

/** Timeout after that the color changes to blue */
#define LIMIT_MS 100

/** Interval of checking the connection to the webserver */
#define INTERVAL_MS 10000

/** Configuration service on http://192.168.4.1 */
static ConfigService configService(80);

/** Used to check the connection to the webserver */
static WiFiClient client;

void led_off()
{
    digitalWrite(0, HIGH);
    digitalWrite(1, HIGH);
    digitalWrite(2, HIGH);
}

void led_green()
{
    digitalWrite(0, LOW);
    digitalWrite(1, HIGH);
    digitalWrite(2, HIGH);
}

void led_red()
{
    digitalWrite(0, HIGH);
    digitalWrite(1, HIGH);
    digitalWrite(2, LOW);
}

void led_blue()
{
    digitalWrite(0, HIGH);
    digitalWrite(1, LOW);
    digitalWrite(2, HIGH);
}


/** Executed once at startup */
void setup()
{    
    // initialize LED outputs
    pinMode(0, OUTPUT);
    pinMode(1, OUTPUT);
    pinMode(2, OUTPUT);
   
    // Run the configuration service for 5 minutes without password.
    configService.begin(5,"WiFi Monitor","");
}


/** Main loop, executed repeatedly */
void loop()
{    
    static unsigned long lastInterval=0;
    if (millis()-lastInterval>INTERVAL_MS)
    {
        lastInterval=millis();
               
        // Are we connected to the AP?
        if (WiFi.status()!=WL_CONNECTED)
        {
            led_off();
        }
        else
        {
            // Connect to the webserver and measure the time
            unsigned long start=millis();
            if (client.connect(eePromData.webserver, 80)) 
            {
                unsigned long end=millis();
                if (end-start > LIMIT_MS)
                {
                    led_blue();
                }    
                else
                {
                    led_green();
                }
            }
            else
            {
                // Connection has failed
                led_red();
            }
            client.stop();
        }
    }

    String s;
    
    // Run the configuration service
    configService.run();
}
